/*
   (c) Copyright 2001-2011  The world wide DirectFB Open Source Community (directfb.org)
   (c) Copyright 2000-2004  Convergence (integrated media) GmbH

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>,
              Andreas Hundt <andi@fischlustig.de>,
              Sven Neumann <neo@directfb.org>,
              Ville Syrjälä <syrjala@sci.fi> and
              Claudio Ciccani <klan@users.sf.net>.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <config.h>

#include "CoreScreen.h"

#include <directfb_util.h>

#include <direct/debug.h>
#include <direct/mem.h>
#include <direct/memcpy.h>
#include <direct/messages.h>

#include <fusion/conf.h>

#include <core/core.h>

D_DEBUG_DOMAIN( DirectFB_CoreScreen, "DirectFB/CoreScreen", "DirectFB CoreScreen" );

/*********************************************************************************************************************/

DFBResult
CoreScreen_SetPowerMode(
                    CoreScreen                                *obj,
                    DFBScreenPowerMode                         mode
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__SetPowerMode( obj, mode );
    }

    return IScreen_Requestor__SetPowerMode( obj, mode );
}

DFBResult
CoreScreen_WaitVSync(
                    CoreScreen                                *obj

)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__WaitVSync( obj );
    }

    return IScreen_Requestor__WaitVSync( obj );
}

DFBResult
CoreScreen_GetVSyncCount(
                    CoreScreen                                *obj,
                    u64                                       *ret_count
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__GetVSyncCount( obj, ret_count );
    }

    return IScreen_Requestor__GetVSyncCount( obj, ret_count );
}

DFBResult
CoreScreen_TestMixerConfig(
                    CoreScreen                                *obj,
                    u32                                        mixer,
                    const DFBScreenMixerConfig                *config,
                    DFBScreenMixerConfigFlags                 *ret_failed
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__TestMixerConfig( obj, mixer, config, ret_failed );
    }

    return IScreen_Requestor__TestMixerConfig( obj, mixer, config, ret_failed );
}

DFBResult
CoreScreen_SetMixerConfig(
                    CoreScreen                                *obj,
                    u32                                        mixer,
                    const DFBScreenMixerConfig                *config
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__SetMixerConfig( obj, mixer, config );
    }

    return IScreen_Requestor__SetMixerConfig( obj, mixer, config );
}

DFBResult
CoreScreen_TestEncoderConfig(
                    CoreScreen                                *obj,
                    u32                                        encoder,
                    const DFBScreenEncoderConfig              *config,
                    DFBScreenEncoderConfigFlags               *ret_failed
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__TestEncoderConfig( obj, encoder, config, ret_failed );
    }

    return IScreen_Requestor__TestEncoderConfig( obj, encoder, config, ret_failed );
}

DFBResult
CoreScreen_SetEncoderConfig(
                    CoreScreen                                *obj,
                    u32                                        encoder,
                    const DFBScreenEncoderConfig              *config
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__SetEncoderConfig( obj, encoder, config );
    }

    return IScreen_Requestor__SetEncoderConfig( obj, encoder, config );
}

DFBResult
CoreScreen_TestOutputConfig(
                    CoreScreen                                *obj,
                    u32                                        output,
                    const DFBScreenOutputConfig               *config,
                    DFBScreenOutputConfigFlags                *ret_failed
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__TestOutputConfig( obj, output, config, ret_failed );
    }

    return IScreen_Requestor__TestOutputConfig( obj, output, config, ret_failed );
}

DFBResult
CoreScreen_SetOutputConfig(
                    CoreScreen                                *obj,
                    u32                                        output,
                    const DFBScreenOutputConfig               *config
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__SetOutputConfig( obj, output, config );
    }

    return IScreen_Requestor__SetOutputConfig( obj, output, config );
}

DFBResult
CoreScreen_GetScreenSize(
                    CoreScreen                                *obj,
                    DFBDimension                              *ret_size
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__GetScreenSize( obj, ret_size );
    }

    return IScreen_Requestor__GetScreenSize( obj, ret_size );
}

DFBResult
CoreScreen_GetLayerDimension(
                    CoreScreen                                *obj,
                    CoreLayer                                 *layer,
                    DFBDimension                              *ret_size
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return IScreen_Real__GetLayerDimension( obj, layer, ret_size );
    }

    return IScreen_Requestor__GetLayerDimension( obj, layer, ret_size );
}

/*********************************************************************************************************************/

static FusionCallHandlerResult
CoreScreen_Dispatch( int           caller,   /* fusion id of the caller */
                     int           call_arg, /* optional call parameter */
                     void         *ptr, /* optional call parameter */
                     unsigned int  length,
                     void         *ctx,      /* optional handler context */
                     unsigned int  serial,
                     void         *ret_ptr,
                     unsigned int  ret_size,
                     unsigned int *ret_length )
{
    CoreScreen *obj = (CoreScreen*) ctx;
    CoreScreenDispatch__Dispatch( obj, caller, call_arg, ptr, length, ret_ptr, ret_size, ret_length );

    return FCHR_RETURN;
}

void CoreScreen_Init_Dispatch(
                    CoreDFB              *core,
                    CoreScreen           *obj,
                    FusionCall           *call
)
{
    fusion_call_init3( call, CoreScreen_Dispatch, obj, core->world );
}

void  CoreScreen_Deinit_Dispatch(
                    FusionCall           *call
)
{
     fusion_call_destroy( call );
}

/*********************************************************************************************************************/


DFBResult
IScreen_Requestor__SetPowerMode( CoreScreen *obj,
                    DFBScreenPowerMode                         mode
)
{
    DFBResult           ret;
    CoreScreenSetPowerMode       *block = (CoreScreenSetPowerMode*) alloca( sizeof(CoreScreenSetPowerMode) );
    CoreScreenSetPowerModeReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );


    block->mode = mode;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_SetPowerMode, block, sizeof(CoreScreenSetPowerMode), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_SetPowerMode ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_SetPowerMode failed!\n", __FUNCTION__ );
         return return_block.result;
    }


    return DFB_OK;
}


DFBResult
IScreen_Requestor__WaitVSync( CoreScreen *obj

)
{
    DFBResult           ret;
    CoreScreenWaitVSync       *block = (CoreScreenWaitVSync*) alloca( sizeof(CoreScreenWaitVSync) );
    CoreScreenWaitVSyncReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );



    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_WaitVSync, block, sizeof(CoreScreenWaitVSync), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_WaitVSync ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_WaitVSync failed!\n", __FUNCTION__ );
         return return_block.result;
    }


    return DFB_OK;
}


DFBResult
IScreen_Requestor__GetVSyncCount( CoreScreen *obj,
                    u64                                       *ret_count
)
{
    DFBResult           ret;
    CoreScreenGetVSyncCount       *block = (CoreScreenGetVSyncCount*) alloca( sizeof(CoreScreenGetVSyncCount) );
    CoreScreenGetVSyncCountReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );



    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_GetVSyncCount, block, sizeof(CoreScreenGetVSyncCount), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_GetVSyncCount ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_GetVSyncCount failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_count = return_block.count;

    return DFB_OK;
}


DFBResult
IScreen_Requestor__TestMixerConfig( CoreScreen *obj,
                    u32                                        mixer,
                    const DFBScreenMixerConfig                *config,
                    DFBScreenMixerConfigFlags                 *ret_failed
)
{
    DFBResult           ret;
    CoreScreenTestMixerConfig       *block = (CoreScreenTestMixerConfig*) alloca( sizeof(CoreScreenTestMixerConfig) );
    CoreScreenTestMixerConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->mixer = mixer;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_TestMixerConfig, block, sizeof(CoreScreenTestMixerConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_TestMixerConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_TestMixerConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_failed = return_block.failed;

    return DFB_OK;
}


DFBResult
IScreen_Requestor__SetMixerConfig( CoreScreen *obj,
                    u32                                        mixer,
                    const DFBScreenMixerConfig                *config
)
{
    DFBResult           ret;
    CoreScreenSetMixerConfig       *block = (CoreScreenSetMixerConfig*) alloca( sizeof(CoreScreenSetMixerConfig) );
    CoreScreenSetMixerConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->mixer = mixer;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_SetMixerConfig, block, sizeof(CoreScreenSetMixerConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_SetMixerConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_SetMixerConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }


    return DFB_OK;
}


DFBResult
IScreen_Requestor__TestEncoderConfig( CoreScreen *obj,
                    u32                                        encoder,
                    const DFBScreenEncoderConfig              *config,
                    DFBScreenEncoderConfigFlags               *ret_failed
)
{
    DFBResult           ret;
    CoreScreenTestEncoderConfig       *block = (CoreScreenTestEncoderConfig*) alloca( sizeof(CoreScreenTestEncoderConfig) );
    CoreScreenTestEncoderConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->encoder = encoder;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_TestEncoderConfig, block, sizeof(CoreScreenTestEncoderConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_TestEncoderConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_TestEncoderConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_failed = return_block.failed;

    return DFB_OK;
}


DFBResult
IScreen_Requestor__SetEncoderConfig( CoreScreen *obj,
                    u32                                        encoder,
                    const DFBScreenEncoderConfig              *config
)
{
    DFBResult           ret;
    CoreScreenSetEncoderConfig       *block = (CoreScreenSetEncoderConfig*) alloca( sizeof(CoreScreenSetEncoderConfig) );
    CoreScreenSetEncoderConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->encoder = encoder;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_SetEncoderConfig, block, sizeof(CoreScreenSetEncoderConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_SetEncoderConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_SetEncoderConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }


    return DFB_OK;
}


DFBResult
IScreen_Requestor__TestOutputConfig( CoreScreen *obj,
                    u32                                        output,
                    const DFBScreenOutputConfig               *config,
                    DFBScreenOutputConfigFlags                *ret_failed
)
{
    DFBResult           ret;
    CoreScreenTestOutputConfig       *block = (CoreScreenTestOutputConfig*) alloca( sizeof(CoreScreenTestOutputConfig) );
    CoreScreenTestOutputConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->output = output;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_TestOutputConfig, block, sizeof(CoreScreenTestOutputConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_TestOutputConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_TestOutputConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_failed = return_block.failed;

    return DFB_OK;
}


DFBResult
IScreen_Requestor__SetOutputConfig( CoreScreen *obj,
                    u32                                        output,
                    const DFBScreenOutputConfig               *config
)
{
    DFBResult           ret;
    CoreScreenSetOutputConfig       *block = (CoreScreenSetOutputConfig*) alloca( sizeof(CoreScreenSetOutputConfig) );
    CoreScreenSetOutputConfigReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( config != NULL );

    block->output = output;
    block->config = *config;

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_SetOutputConfig, block, sizeof(CoreScreenSetOutputConfig), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_SetOutputConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_SetOutputConfig failed!\n", __FUNCTION__ );
         return return_block.result;
    }


    return DFB_OK;
}


DFBResult
IScreen_Requestor__GetScreenSize( CoreScreen *obj,
                    DFBDimension                              *ret_size
)
{
    DFBResult           ret;
    CoreScreenGetScreenSize       *block = (CoreScreenGetScreenSize*) alloca( sizeof(CoreScreenGetScreenSize) );
    CoreScreenGetScreenSizeReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( ret_size != NULL );


    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_GetScreenSize, block, sizeof(CoreScreenGetScreenSize), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_GetScreenSize ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_GetScreenSize failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_size = return_block.size;

    return DFB_OK;
}


DFBResult
IScreen_Requestor__GetLayerDimension( CoreScreen *obj,
                    CoreLayer                                 *layer,
                    DFBDimension                              *ret_size
)
{
    DFBResult           ret;
    CoreScreenGetLayerDimension       *block = (CoreScreenGetLayerDimension*) alloca( sizeof(CoreScreenGetLayerDimension) );
    CoreScreenGetLayerDimensionReturn  return_block;

    D_DEBUG_AT( DirectFB_CoreScreen, "IScreen_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( layer != NULL );
    D_ASSERT( ret_size != NULL );

    block->layer_id = CoreLayer_GetID( layer );

    ret = (DFBResult) CoreScreen_Call( obj, FCEF_NONE, _CoreScreen_GetLayerDimension, block, sizeof(CoreScreenGetLayerDimension), &return_block, sizeof(return_block), NULL );
    if (ret) {
        D_DERROR( ret, "%s: CoreScreen_Call( CoreScreen_GetLayerDimension ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_block.result) {
         D_DERROR( return_block.result, "%s: CoreScreen_GetLayerDimension failed!\n", __FUNCTION__ );
         return return_block.result;
    }

    *ret_size = return_block.size;

    return DFB_OK;
}

/*********************************************************************************************************************/

static DFBResult
__CoreScreenDispatch__Dispatch( CoreScreen *obj,
                                FusionID      caller,
                                int           method,
                                void         *ptr,
                                unsigned int  length,
                                void         *ret_ptr,
                                unsigned int  ret_size,
                                unsigned int *ret_length )
{
    D_UNUSED
    DFBResult ret;


    switch (method) {
        case _CoreScreen_SetPowerMode: {
            D_UNUSED
            CoreScreenSetPowerMode       *args        = (CoreScreenSetPowerMode *) ptr;
            CoreScreenSetPowerModeReturn *return_args = (CoreScreenSetPowerModeReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_SetPowerMode\n" );

            return_args->result = IScreen_Real__SetPowerMode( obj, args->mode );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenSetPowerModeReturn);

            return DFB_OK;
        }

        case _CoreScreen_WaitVSync: {
            D_UNUSED
            CoreScreenWaitVSync       *args        = (CoreScreenWaitVSync *) ptr;
            CoreScreenWaitVSyncReturn *return_args = (CoreScreenWaitVSyncReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_WaitVSync\n" );

            return_args->result = IScreen_Real__WaitVSync( obj );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenWaitVSyncReturn);

            return DFB_OK;
        }

        case _CoreScreen_GetVSyncCount: {
            D_UNUSED
            CoreScreenGetVSyncCount       *args        = (CoreScreenGetVSyncCount *) ptr;
            CoreScreenGetVSyncCountReturn *return_args = (CoreScreenGetVSyncCountReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_GetVSyncCount\n" );

            return_args->result = IScreen_Real__GetVSyncCount( obj, &return_args->count );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenGetVSyncCountReturn);

            return DFB_OK;
        }

        case _CoreScreen_TestMixerConfig: {
            D_UNUSED
            CoreScreenTestMixerConfig       *args        = (CoreScreenTestMixerConfig *) ptr;
            CoreScreenTestMixerConfigReturn *return_args = (CoreScreenTestMixerConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_TestMixerConfig\n" );

            return_args->result = IScreen_Real__TestMixerConfig( obj, args->mixer, &args->config, &return_args->failed );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenTestMixerConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_SetMixerConfig: {
            D_UNUSED
            CoreScreenSetMixerConfig       *args        = (CoreScreenSetMixerConfig *) ptr;
            CoreScreenSetMixerConfigReturn *return_args = (CoreScreenSetMixerConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_SetMixerConfig\n" );

            return_args->result = IScreen_Real__SetMixerConfig( obj, args->mixer, &args->config );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenSetMixerConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_TestEncoderConfig: {
            D_UNUSED
            CoreScreenTestEncoderConfig       *args        = (CoreScreenTestEncoderConfig *) ptr;
            CoreScreenTestEncoderConfigReturn *return_args = (CoreScreenTestEncoderConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_TestEncoderConfig\n" );

            return_args->result = IScreen_Real__TestEncoderConfig( obj, args->encoder, &args->config, &return_args->failed );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenTestEncoderConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_SetEncoderConfig: {
            D_UNUSED
            CoreScreenSetEncoderConfig       *args        = (CoreScreenSetEncoderConfig *) ptr;
            CoreScreenSetEncoderConfigReturn *return_args = (CoreScreenSetEncoderConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_SetEncoderConfig\n" );

            return_args->result = IScreen_Real__SetEncoderConfig( obj, args->encoder, &args->config );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenSetEncoderConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_TestOutputConfig: {
            D_UNUSED
            CoreScreenTestOutputConfig       *args        = (CoreScreenTestOutputConfig *) ptr;
            CoreScreenTestOutputConfigReturn *return_args = (CoreScreenTestOutputConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_TestOutputConfig\n" );

            return_args->result = IScreen_Real__TestOutputConfig( obj, args->output, &args->config, &return_args->failed );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenTestOutputConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_SetOutputConfig: {
            D_UNUSED
            CoreScreenSetOutputConfig       *args        = (CoreScreenSetOutputConfig *) ptr;
            CoreScreenSetOutputConfigReturn *return_args = (CoreScreenSetOutputConfigReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_SetOutputConfig\n" );

            return_args->result = IScreen_Real__SetOutputConfig( obj, args->output, &args->config );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenSetOutputConfigReturn);

            return DFB_OK;
        }

        case _CoreScreen_GetScreenSize: {
            D_UNUSED
            CoreScreenGetScreenSize       *args        = (CoreScreenGetScreenSize *) ptr;
            CoreScreenGetScreenSizeReturn *return_args = (CoreScreenGetScreenSizeReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_GetScreenSize\n" );

            return_args->result = IScreen_Real__GetScreenSize( obj, &return_args->size );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenGetScreenSizeReturn);

            return DFB_OK;
        }

        case _CoreScreen_GetLayerDimension: {
    CoreLayer *layer = NULL;
            D_UNUSED
            CoreScreenGetLayerDimension       *args        = (CoreScreenGetLayerDimension *) ptr;
            CoreScreenGetLayerDimensionReturn *return_args = (CoreScreenGetLayerDimensionReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_CoreScreen, "=-> CoreScreen_GetLayerDimension\n" );

            ret = (DFBResult) CoreLayer_Lookup( core_dfb, args->layer_id, caller, &layer );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up layer by ID %u failed!\n", __FUNCTION__, args->layer_id );
                 return_args->result = ret;
                 return DFB_OK;
            }

            return_args->result = IScreen_Real__GetLayerDimension( obj, layer, &return_args->size );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(CoreScreenGetLayerDimensionReturn);

            if (layer)
                CoreLayer_Unref( layer );

            return DFB_OK;
        }

    }

    return DFB_NOSUCHMETHOD;
}
/*********************************************************************************************************************/

DFBResult
CoreScreenDispatch__Dispatch( CoreScreen *obj,
                                FusionID      caller,
                                int           method,
                                void         *ptr,
                                unsigned int  length,
                                void         *ret_ptr,
                                unsigned int  ret_size,
                                unsigned int *ret_length )
{
    DFBResult ret;

    D_DEBUG_AT( DirectFB_CoreScreen, "CoreScreenDispatch::%s()\n", __FUNCTION__ );

    Core_PushIdentity( caller );

    ret = __CoreScreenDispatch__Dispatch( obj, caller, method, ptr, length, ret_ptr, ret_size, ret_length );

    Core_PopIdentity();

    return ret;
}
